LIBRARY UNZIP32 INITINSTANCE LONGNAMES
PROTMODE
DESCRIPTION 'Info-ZIP UnZip API - Copyright Scott Maxwell 1994, Info-ZIP 1995-1997'
DATA MULTIPLE NONSHARED
EXPORTS
     UzpVersion		@1
     UzpUnzipToMemory	@2
     UzpFileTree	@3
     UzpMain		@4
     UzpAltMain		@5
     UzpVersion2	@6
     UZDROPFUNCS	= UZDropFuncs      @2000
     UZLOADFUNCS	= UZLoadFuncs      @2001
     UZFILETREE		= UZFileTree       @2002
     UZUNZIPTOVAR	= UZUnZipToVar     @2003
     UZUNZIPTOSTEM	= UZUnZipToStem    @2004
     UZVER		= UZVer            @2005
     UZUNZIP		= UZUnZip	   @2007
